
	.GLOBAL TMSCODE	

; Programmer: Jed Margolin
;
; Date: 2/9/90
; PC Version
; Hardware: DSK
;-----------------------------------------------------------------------------
; Table memory:		
;
; Ram 256 words, 128 words direct, 2 pages
;-----------------------------------------------------------------------------
	.TITLE	"Ports and Zero Page Memory"
	.PAGE
; 320 Ports
PIN:		.SET	PA0 	; R, Interface Input

POUT:		.SET	PA0 	; W, Interface Output

STAT:		.SET	PA1	; R, Status Flags
				; 	d15 = 320FLAG
				; 	d14 = 68FLAG
				; 	d13 = CMD
				; 	d12 = M

XFLAG		.SET	PA1	; W, Extra Flag

; Zero Page RAM Variables
;		.=0000
	.bss 	zero,1
 	.bss	one,1
 	.bss	temp1,1
	.bss 	temp2,1
	.bss	padr,1
 	.bss	chksum,1
	.bss	speed,1
	.bss	cntr1,1
	.bss	cntr2,1
	.bss	allon,1
	.bss	bmask,1
	.bss	cmask,1
	.bss	mptr,1	
	.bss	input,1
	.bss	status,1
	.bss	buffer,16

	.GLOBAL	zero,one,temp1,temp2,padr,chksum,speed,cntr1,cntr2
	.GLOBAL	allon,bmask,cmask,mptr,input,status,buffer
;--------------------------------------------------------------------------
;	Startup
	.PAGE
	.sect "TMSCODE"
RESET:	B	START		; Reset Vector
INTR:	B	INT		; Interrupt Vector (Not used in this program)
;---------------------------------
; Data
CKSUM:	.WORD	(-45h+27h)&00FFh  ; makes the checksum turn out right; should
				; be $27 (Low byte only)
MSGADR:	.WORD	MSG		; Address pointer to Message
VERSION: .WORD	0011h		; version 1.1

; Table for Command Routines
CMDTBL:	.WORD	CMD0,CMD1,CMD2,CMD3,CMD4,CMD5,CMD6,CMD7
	.WORD	CMD8,CMD9,CMD10,CMD11,CMD12,CMD13,CMD14,CMD15,CMD16,CMD17
	.WORD	CMD18,CMD19,CMD20,CMD21,CMD22,CMD23,CMD24,CMD25,CMD26,CMD27

	.GLOBAL	CMD0,CMD1,CMD2,CMD3,CMD4,CMD5,CMD6,CMD7
	.GLOBAL	CMD8,CMD9,CMD10,CMD11,CMD12,CMD13,CMD14,CMD15,CMD16,CMD17
	.GLOBAL	CMD18,CMD19,CMD20,CMD21,CMD22,CMD23,CMD24,CMD25,CMD26,CMD27
	.GLOBAL COMMAND,SPIN

; CMD0		; just echo the input port (Slow Echo)	
; CMD1		; just echo the input port (Fast Echo)	
; CMD2		; Checksum
; CMD3		; Version Number
; CMD4		; RAM Test will go here 
; CMD5		; SPIN (XFLAG Tone)
; CMD6		; Test 
; CMD7		; Idle and wait for Command 
; CMD8 - CMD27 ; Real Programs
;---------------------------------
INT:	B	SPIN 		; Interrupt routine would go here
;---------------------------------
; From Reset
START:	LDPK	0		; Data Memory Page 0

	LACK	1
	SACL	one

; XMIT Test?
	IN	status,STAT
 	LAC	one,12		; d12 = M
	AND	status
	BZ	XMIT
;=============================================================================
	.TITLE	"Commands"
	.PAGE
COMMAND:
	BIOZ	COMMAND		; Is the interface ready?

; Command ?
	LAC	one,13		; d13 = CMD
	AND	status
	BZ	SPIN		; no command, this is an error, so for now Spin
				; for development only
; Get the command and go to it
	IN	input,PIN
	LACK	0FFh			; mask
	AND	input
	SACL	temp1			; command is in low byte

	LACK	CMDTBL			; address of CMDTBL
	ADD	temp1			; table index (command)
	TBLR	mptr			; mptr points to the command routine

	LAC	mptr
	CALA				; Address comes from ACC

; If it ever comes back, just spin (This is an error, for development only)
	B	SPIN
;==============================================================================
; Toggles the XFLAG at 400Hz
SPIN:	ZAC
	SACL	zero
	SUB	one
	SACL	allon
SPINLP:	CALL	MWAIT
	OUT	zero,XFLAG
	CALL	MWAIT
	OUT	allon,XFLAG
	B	SPINLP
;==============================================================================
; Command 0 is to just slow echo. Start by echoing 0000 
CMD0:	OUT	input,POUT
CM0A:	IN	status,STAT
	LAC	one,14
	AND	status
	BZ	CM0A

; we have a flag
	IN	input,PIN
	OUT	input,POUT
	B	CM0A	
;-------------------------------------------------------------------------------
; Command 1. Fast Echo
CMD1:	BIOZ	CMD1
	IN	temp1,PIN
	OUT	temp1,POUT
	B	CMD1
;-------------------------------------------------------------------------------
; Command 2. Do checksum
CMD2:	CALL	CHECKSUM
	B	COMMAND
;---------------------------------
; Command 3. Version Number
CMD3:	LACK	VERSION
	TBLR	temp1
	OUT	temp1,POUT
	B	COMMAND
;---------------------------------
; Command 4. RAM Test will go here
CMD4:	B	SPIN
;---------------------------------
; Command 5. Test SPIN
CMD5:	B	SPIN
;---------------------------------
; Command 6. Test (Reserved)
CMD6:	B	SPIN
;---------------------------------
; Command 7. Test (Reserved)
CMD7:	B	COMMAND
;=============================================================================
	.TITLE	"Checksum"
	.PAGE
CHECKSUM:
	LDPK	0		; Data Memory Page 0

	LACK	1
	SACL	one

	ZAC
	SACL	zero
	SACL	chksum

	LAC	one,12
	SUB	one
	SACL	padr		; padr = 4095

	LACK	0FFh
	SACL	bmask		; for masking low bytes

; Read program memory
CKLOOP:	LAC	padr
	TBLR	temp1

; Add to checksum. 
; Separate the bytes so it will be compatible with DIO checksums
	LAC	temp1
	AND	bmask		; 00FF
	ADD	chksum 
	SACL	chksum 		; we have done the low byte

	LAC	temp1,8
	SACH	temp1		; temp1 now contains the high byte in d0-d7
	LAC	temp1
	AND	bmask
	ADD 	chksum 
	SACL	chksum

; Decrement pointer
	ZALS	padr
	SUB	one
	SACL	padr

; Are we done?
	BGEZ	CKLOOP

; Report
	OUT	chksum,POUT
	RET
;----------------------------------------------------------------------
	.TITLE	"Transmit"
	.PAGE

MINIT:	LDPK	0			; Speed WPM = 2.4 x dots/sec
	LARP	AR0

	LACK	1
	SACL	one

	LAC	one,5			
	SACL	speed

	ZAC
	SACL	zero
	SACL	chksum

	SUB	one
	SACL	allon		; FFFF

	RET
;---------------------------
MWAIT:	LAC	one,11
	SACL	cntr1
	ZALS	cntr1
WLOOP:	SUB	one
	BNZ	WLOOP
	RET
;---------------------------
DIT:	LAC	speed
	SACL	cntr2

DLOOP:	OUT	zero,XFLAG
	CALL	MWAIT
	OUT	allon,XFLAG
	CALL	MWAIT
	
	ZALS	cntr2
	SUB	one
	SACL	cntr2
	BNZ	DLOOP
	RET
;---------------------------
DAH:	LT	speed
	MPYK	3
	PAC
	SACL	cntr2
	B	DLOOP
;---------------------------
ELSPACE:
	LAC	speed			; speed
	SACL	cntr2
ELOOP:	OUT	zero,XFLAG
	CALL	MWAIT
	OUT	zero,XFLAG
	CALL	MWAIT

	ZALS	cntr2
	SUB	one
	SACL	cntr2
	BNZ	ELOOP

	RET
	.PAGE
;---------------------------
CHSPACE:
	CALL	ELSPACE	     ; character spaces already have one element space
	CALL	ELSPACE
	CALL	ELSPACE
	RET
;---------------------------
WORDSPACE:
	LARP	AR0		; use AR0 as the counter
	LARK	AR0,8		
WSPLP:	CALL	ELSPACE
	BANZ	WSPLP		; also decrements AR0
	RET
;=============================================================================
XMIT:	LDPK	0			; Speed WPM = 2.4 x dots/sec
	LARP	AR0

	LACK	1
	SACL	one

	LAC	one,5			
	SACL	speed

	ZAC
	SACL	zero
	SACL	chksum

	SUB	one
	SACL	allon			; FFFF

; Setup the counter mask
	LACK	7
	SACL	cmask

; Setup the message counter
	LACK	MSGADR
	TBLR	mptr			; mptr points to the message

; get the next word
XLP1:	LAC	mptr
	TBLR	temp1			; next data

; check if it is a word space
	LAC	temp1
	BNZ	XLP2
	CALL	WORDSPACE
	B	XLP7

; check if it is the end
XLP2:	SUB	allon			; 0FFFF = termination character
	BZ	MOUT			; we are done

; mask off the low 3 bits to be the element counter	
	LAC	temp1
	SUB	one
	AND	cmask
	SACL	temp2
	LAR	AR0,temp2		; AR0 is the element counter

XLP3:	LAC	temp1,1
	SACL	temp1
	BLZ	XLP4			; minus = dah
	CALL	DIT
	B	XLP5

XLP4:	CALL	DAH

XLP5:	CALL	ELSPACE			; element space

XLP6:	BANZ	XLP3		; next element in the word, also decrements AR0

	CALL	CHSPACE		; character space 

; we are done with the character, increment the message pointer
XLP7:	ZALS	mptr
	ADD	one
	SACL	mptr
	B	XLP1			; next word

MOUT:	RET
;=============================================================================
MSG:	.WORD	0004|0A000h		; C	-.-.	1010	4
	.WORD	0003|0E000h		; O	---	111	3
	.WORD	0004|06000h		; P	.--.	0110	4
	.WORD	0004|0B000h		; Y	-.--	1011	4
	.WORD	0003|04000h		; R	.-.	010	3
	.WORD	0002|00000h		; I	..	00	2
	.WORD	0003|0C000h		; G	--.	110	3
	.WORD	0004|00000h		; H	....	0000	4
	.WORD	0001|08000h		; T	-	1	1
	.WORD	0000			; 			0
	.WORD	0002|4000h		; A	.-	01	2
	.WORD	0001|8000h		; T	-	1	1
	.WORD	0002|4000h		; A	.-	01	2
	.WORD	0003|4000h		; R	.-.	010	3
	.WORD	0002|0000h		; I	..	00	2
	.WORD	0000			; 			0
	.WORD	0003|0C000h		; G	--.	110	3
	.WORD	0002|04000h		; A	.-	01	2
	.WORD	0002|0C000h		; M	--	11	2
	.WORD	0001|00000h		; E	.	0	1
	.WORD	0003|00000h		; S	...	000	3
	.WORD	0000			; 			0
	.WORD	0005|07800h		; 1	.----	01111	5
	.WORD	0005|0F000h		; 9	----.	11110	5
	.WORD	0005|0F000h		; 9	----.	11110	5
	.WORD	0005|0F800h		; 0	-----	11111	5
	.WORD	0FFFFh
;=============================================================================
	.END
