	TTL 'VGMC'

* TomCat VG Macros
* Original Macros programmed by ED LOGG, 8 October 1979
* Updated/Revised/Enhanced for Star Wars in June '83, by GREGR and NORM A.
* Rejuberated for ASM68 by: Jed Margolin, April 1985.

* The address offset of the alphanumerics in the Vector Generator address
* space will be called VGANLOC. If, for example, alphanumerics go in 
* the last the 4K of VG's 16K address then VGANLOC = $3000  .
* For a subroutine at label CHAR_ADD, the JSRL address will be:
*	CHAR_ADD-VGIMAGE+VGANLOC
*
* Sample to try test data: 
* TEST.ASM:
*		TTL 'TEST.ASM'	
*	VGANLOC EQU	$3000	*VG Alphanumerics go in last 4K (of VG's 
*				16K address space)	
*	VGIMAGE EQU	$5000	*Image will be ORG'ed at $5000 for now
*	
*	RED	EQU	$4
*	GREEN	EQU	$2
*	BLUE	EQU	$1
*	WHT	EQU	$7
*
*		ORG	VGIMAGE
*		CNTR
*		COLOR	RED,$40
*		SCAL	2,0		*binary scale=2, linear scale=$40	
*		JSRL	SUBR
*		ALPHA	T,E,S,T,~
*		HALT
*	SUBR	VCTR	$100,$100,OFF
*		VCTR	-$100,0,BRITE
*		RTSL
*
*		END		
*------------------------------------------------------------------------------
*---
*	CNTR - Center the beam in the middle of the screen.
*---
CNTR	MACRO 
	DC.W	$8040			*VGCNTR
	ENDM
*---
*	HALT - When executed by the Vector Generator sets the VG Halt Flag
*---
HALT	MACRO	
	DC.W	$2000
	ENDM
*---
*	JSRL - JUMP SUBROUTINE INSTRUCTION
*	The lower thirteen bits specify a word address relative to the 
*	beginning of the Vector Generator addresable instruction space
*	(relative byte divided by 2). The total address area is 16K bytes
*	Only three levels deep work. The forth overlays the first, and so on.
*
*	This will be a VG memory image residing in 68010 program memory
*	at absolute origin VGIMAGE. 
*
*	The address offset of the Alphanumerics in the Vector Generator address
*	space will be called VGANLOC. If, for example, alphanumerics go in 
*	the last the 4K of VG's 16K address then VGANLOC = $3000  .
* 	For a subroutine at label CHAR_ADD, the JSRL address will be:
*		CHAR_ADD-VGIMAGE+VGANLOC
*
*---	
JSRL	MACRO
*	DC.W	((\1-VGIMAGE+VGANLOC)/2)&$1FFF!$0A000
	DC.W	((\1-VIMAGE+VGLOC)/2)&$1FFF!$0A000
	ENDM
*---	
* JSRLs for Alphanumerics
CJSRL	MACRO	
	DC.W	((\1-VGIMAGE+VGANLOC)/2)&$1FFF!$0A000
	ENDM
*---
* 	COLOR sets the three color bits (RGB) and the Linear Intensity (0 to FF)
* 	80 is the highest normal intensity; 81 thru FF produce gun saturation
* 	which results in beam spreading and defocussing for special effects.
*
* 	Format: COLOR COLOR,ZSCALE
*---
COLOR	MACRO
	DC.W	$6000!((\1&$7)*$100)!(\2&$0FF)
	ENDM
*---
*	SCAL - SET VECTOR GENERATOR SCALE
*	The first parameter is the power of 2 scale. A value of 2 means
*	full size, 3=1/2 size, 4=1/4 size, ...,7=1/32 size. The second
*	parameter is the Linear Scale (Optional). A value of 0 means
*	full size, 80=1/2 size, FF=1/256 size. Linear scaling does not
*	change the vector drawing time, it just makes it smaller (and
*	to some extent, brighter).
*
* 	Format:	SCAL BIN,LIN
*---
SCAL	MACRO
	DC.W	$7000!((\1&$7)*$100)!(\2&$0FF)
	ENDM
*---
*	RTSL - RETURN FROM SUBROUTINE 
*	This instruction is analagous to an RTS instruction for a
*	microprocessor. The VG is capable of only three levels of
*	subroutine calls (JSRL).
*---
RTSL	MACRO	
	DC.W	$0C000
	ENDM
*---
*	JMPL - JUMP TO ROUTINE
*	The lower thirteen bits reference an address the same as in
*	a JSRL instruction. There is no return address saved on the
*	VG stack. Refer to JSRL for an explanation of the addressing.
*---
JMPL	MACRO
*	DC.W	((\1-VGIMAGE+VGANLOC)/2)&$1FFF!$0E000
	DC.W	((\1-VIMAGE+VGLOC)/2)&$1FFF!$0E000
	ENDM
*---
*	VCTR - DRAW VECTOR INSTRUCTION
*	This instruction draws a vector on the display relative to
*	the previous beam position. DX is the change in Beam X
*	position, DY is the change in Beam Y position, ZZ specifies
*	the beam intensity (0 thru 7; 0 is no intensity, 7 is brightest
*	intensity). This intensity is scaled by ZSCALE set by the COLOR
*	instruction.
*---
*	Short Vectors are drawn twice the length of the unadjusted dx and
*	dy values embedded in the SVECT instruction. Therefore, the
*	Macro will take the DX, DY values in the argument and divide
*	them by two. When they are executed by the hardware they will
*	be doubled back to normal size.
*
*	Since there are 5 bits, the maximum unadjusted range is -16 to +15.
*	With scaling, this becomes -32 to +30, with even values only.
*	A vector that does not meet these requirements will be drawn as a
*	standard (long) vector. Also, short vectors may not have both
*	DX and DY be zero.
*
*	Format:	 VCTR DX,DY,ZZ
*---
VCTR	MACRO
DX	SET	\1	*For temporary storage
DY	SET	\2	*For temporary storage
	IFLT DX		*If DX is negative, make it positive and subtract 2
DX	  SET (-\1)-2	*so that -32 will become +30 instead of +32 which
	ENDC		*wouldn't fit.
	IFLT DY		*Same for DY
DY	  SET (-\2)-2
	ENDC
DXY	SET DX!DY	*OR the positive values together
	IFNE (DX+DY)	*If the result is 0, fall through, become a Long Vector
	  IFEQ DXY&$0FFE1 *If .LT. -32 or .GT. 32 or odd, become a Long Vector
	    DC.B  $40!(((\2)/2)&$1F),(((\3)&7)*$20)!(((\1)/2)&$1F)
	    MEXIT
	  ENDC
	ENDC
	DC.W 	(\2)&$1FFF,(((\3)&7)*$2000)!((\1)&$1FFF) *Long Vector
	ENDM
*---
*	ALPHA - Constructs a JSRL instruction for every character
*	in the calling list. A Label 'CHAR_X' must exist for every
*	unique character 'X' in the calling list. The list must end
*	with a '~'. Only eight characters are allowed per line (in
*	addition to the delimiter).
*
*	Format:	ALPHA	P,L,A,Y,E,R,~	(example)
*---
ALPHA	MACRO
	IFEQ	'\1'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\1

	IFEQ	'\2'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\2

	IFEQ	'\3'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\3

	IFEQ	'\4'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\4

	IFEQ	'\5'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\5

	IFEQ	'\6'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\6

	IFEQ	'\7'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\7

	IFEQ	'\8'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\8

	IFEQ	'\9'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\9

	ENDM
*---
*	ALPHAV - Same as APLHA, but places the characters vertically from top 
*	to bottom. See ALPHA for more info.
*
*	Format:	ALPHAV	P,L,A,Y,E,R,~	(example)
*---
ALPHAV	MACRO
	IFEQ	'\1'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\1
	VCTR	-24,-36,0

	IFEQ	'\2'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\2
	VCTR	-24,-36,0

	IFEQ	'\3'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\3
	VCTR	-24,-36,0

	IFEQ	'\4'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\4
	VCTR	-24,-36,0

	IFEQ	'\5'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\5
	VCTR	-24,-36,0

	IFEQ	'\6'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\6
	VCTR	-24,-36,0

	IFEQ	'\7'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\7
	VCTR	-24,-36,0

	IFEQ	'\8'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\8
	VCTR	-24,-36,0

	IFEQ	'\9'-'~'
	MEXIT
	ENDC	
	CJSRL	CHAR_\9
	VCTR	-24,-36,0

	ENDM
*---
*Generate Symbols (VJ_\1) with the value of the instruction  JSRL \1
VJ	MACRO	
*VJ_\1	EQU	((\1-VGIMAGE+VGANLOC)/2)&$1FFF!$0A000
VJ_\1	EQU	((\1-VIMAGE+VGLOC)/2)&$1FFF!$0A000
	ENDM

*---
